//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2011, John Haddon. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//      * Redistributions of source code must retain the above
//        copyright notice, this list of conditions and the following
//        disclaimer.
//
//      * Redistributions in binary form must reproduce the above
//        copyright notice, this list of conditions and the following
//        disclaimer in the documentation and/or other materials provided with
//        the distribution.
//
//      * Neither the name of John Haddon nor the names of
//        any other contributors to this software may be used to endorse or
//        promote products derived from this software without specific prior
//        written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
//  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef GAFFERUI_MODIFIABLEEVENT_H
#define GAFFERUI_MODIFIABLEEVENT_H

#include "GafferUI/Event.h"

namespace GafferUI
{

/// A base class for events for which the keyboard
/// modifier keys are relevant.
struct GAFFERUI_API ModifiableEvent : public Event
{
	/// An enum to represent the modifier keys. We deliberately ignore
	/// platform specific modifiers like the Apple and Windows keys to
	/// provide a consistent set of shortcuts across platforms.
	enum Modifiers
	{
		None = 0,
		Shift = 1,
		Control = 2,
		Alt = 4,
		ShiftControl = Shift | Control,
		ShiftAlt = Shift | Alt,
		ControlAlt = Control | Alt,
		All = Shift | Control | Alt
	};

	ModifiableEvent( Modifiers m = None ) : modifiers( m ) {};

	/// The state of the modifier keys.
	Modifiers modifiers;
};

} // namespace GafferUI

#endif // GAFFERUI_MODIFIABLEEVENT_H
