##########################################################################
#
#  Copyright (c) 2013, Image Engine Design Inc. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#
#      * Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      * Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials provided with
#        the distribution.
#
#      * Neither the name of John Haddon nor the names of
#        any other contributors to this software may be used to endorse or
#        promote products derived from this software without specific prior
#        written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
#  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##########################################################################

import Gaffer
import GafferUI
import GafferUITest

class PlaybackTest( GafferUITest.TestCase ) :

	def testAcquire( self ) :

		c1 = Gaffer.Context()
		c2 = Gaffer.Context()

		p1a = GafferUI.Playback.acquire( c1 )
		p1b = GafferUI.Playback.acquire( c1 )

		self.assertTrue( p1a is p1b )

		p2a = GafferUI.Playback.acquire( c2 )
		p2b = GafferUI.Playback.acquire( c2 )

		self.assertTrue( p2a is p2b )
		self.assertTrue( p2a is not p1a )

		s = Gaffer.ScriptNode()
		p3a = GafferUI.Playback.acquire( s.context() )
		p3b = GafferUI.Playback.acquire( s.context() )

		self.assertTrue( p3a is p3b )

	def testCurrentFrameIsSavedInScript( self ) :

		s = Gaffer.ScriptNode()
		p = GafferUI.Playback.acquire( s.context() )
		p.incrementFrame()

		s2 = Gaffer.ScriptNode()
		s2.execute( s.serialise() )
		self.assertEqual( s2.context().getFrame(), s.context().getFrame() )

if __name__ == "__main__":
	unittest.main()
